/****************************************************************/
/*                                                              */
/*      arraydlg.cpp                                            */
/*                                                              */
/*      This file is a part of kpl - a program for graphical    */
/*      presentation of data sets and functions.                */
/*                                                              */
/*      Copyright (C) 1999 Werner Stille                        */
/*      <stille@uni-freiburg.de>                                */
/*                                                              */
/*      Released under the GPL; see file LICENSE for details.   */
/*                                                              */
/****************************************************************/

#include "arraydlg.h"
#include "kpl.h"
#include "symboldlg.h"
#include "utils.h"
#include <kapp.h>
#include <kmsgbox.h>
#include <kfiledialog.h>
#include <kseparator.h>
#include <kcolorbtn.h>
#include <qcheckbox.h>
#include <qspinbox.h>

#ifndef min
  #define min(x,y) ((x) < (y) ? (x) : (y))
#endif
#ifndef max
  #define max(x,y) ((x) > (y) ? (x) : (y))
#endif

ArrayDlg::ArrayDlg(QWidget *parent, KPl *kpa, PARRAYDATA ad0) :
 QDialog(parent, 0, TRUE)
{
  QString s;

  kp = kpa;
  ad = ad0;
  adt = 0;
  setCaption(i18n("Array"));
  l1 = new QLabel(i18n("File"), this);
  l1->setGeometry(10, 12, 84, 22);
  fileName = new QLineEdit(this);
  fileName->setGeometry(100, 12, 244, 22);
  fileName->setText(ad->path);
  chFile = new QPushButton("...", this);
  chFile->setGeometry(355, 11, 25, 25);
  connect(chFile, SIGNAL(clicked()), SLOT(slotFile()));
  l2 = new QLabel(i18n("x column"), this);
  l2->setGeometry(10, 46, 86, 22);
  ix = new QSpinBox(0, 0, 1, this);
  ix->setGeometry(100, 46, 50, 22);
  l3 = new QLabel(i18n("y column"), this);
  l3->setGeometry(230, 46, 86, 22);
  iy = new QSpinBox(0, 0, 1, this);
  iy->setGeometry(330, 46, 50, 22);
  l4 = new QLabel(i18n("Error column"), this);
  l4->setGeometry(10, 80, 86, 22);
  ie = new QSpinBox(0, 0, 1, this);
  ie->setGeometry(100, 80, 50, 22);
  err = new QCheckBox(i18n("Error bar"), this);
  err->setGeometry(170, 80, 120, 22);
  err->setChecked(ad->errbars);
  l5 = new QLabel(i18n("Start index"), this);
  l5->setGeometry(10, 114, 86, 22);
  iStart = new QSpinBox(0, 0, 1, this);
  iStart->setGeometry(100, 114, 50, 22);
  l6 = new QLabel(i18n("Number of points"), this);
  l6->setGeometry(230, 114, 98, 22);
  n = new QSpinBox(0, 0, 1, this);
  n->setGeometry(330, 114, 50, 22);
  l7 = new QLabel(i18n("x normalization"), this);
  l7->setGeometry(10, 148, 88, 22);
  efx = new QLineEdit(this);
  efx->setGeometry(100, 148, 50, 22);
  efx->setText(s.sprintf("%7g", ad->fx));
  l8 = new QLabel(i18n("y normalization"), this);
  l8->setGeometry(230, 148, 88, 22);
  efy = new QLineEdit(this);
  efy->setGeometry(330, 148, 50, 22);
  efy->setText(s.sprintf("%7g", ad->fy));
  l9 = new QLabel(i18n("Symbol"), this);
  l9->setGeometry(10, 182, 86, 22);
  symb = new QSpinBox(-17, 9, 1, this);
  symb->setGeometry(100, 182, 50, 22);
  symb->setValue(ad->symb);
  chSymb = new QPushButton("...", this);
  chSymb->setGeometry(162, 181, 25, 25);
  connect(chSymb, SIGNAL(clicked()), SLOT(slotSymbol()));
  l10 = new QLabel(i18n("Color"), this);
  l10->setGeometry(230, 182, 86, 22);
  colData = new KColorButton(this, 0);
  colData->setGeometry(330, 181, 50, 25);
  colData->setColor(ad->color);
  hl = new KSeparator(KSeparator::HLine, this);
  hl->setGeometry(10, 216, 370, 0);
  ok = new QPushButton(i18n("OK"), this);
  ok->setGeometry(10, 228, 85, 25);
  ok->setDefault(true);
  connect(ok, SIGNAL(clicked()), SLOT(slotOK()));
  apply = new QPushButton(i18n("Apply"), this);
  apply->setGeometry(105, 228, 85, 25);
  connect(apply, SIGNAL(clicked()), SLOT(slotApply()));
  cancel = new QPushButton(i18n("Cancel"), this);
  cancel->setGeometry(200, 228, 85, 25);
  connect(cancel, SIGNAL(clicked()), SLOT(slotCancel()));
  help = new QPushButton(i18n("Help"), this);
  help->setGeometry(295, 228, 85, 25);
  connect(help, SIGNAL(clicked()), SLOT(slotHelp()));
  adjustSize();
  setFixedSize(size());
  setEditArrayControls(ad);
}

ArrayDlg::~ArrayDlg()
{
  delete l1;
  delete l2;
  delete l3;
  delete l4;
  delete l5;
  delete l6;
  delete l7;
  delete l8;
  delete l9;
  delete l10;
  delete efx;
  delete efy;
  delete fileName;
  delete ix;
  delete iy;
  delete ie;
  delete iStart;
  delete n;
  delete symb;
  delete err;
  delete colData;
  delete chFile;
  delete chSymb;
  delete hl;
  delete apply;
  delete ok;
  delete cancel;
  delete help;
}

void ArrayDlg::setEditArrayControls(PARRAYDATA ad)
{
  int iColMax = max(0, ad->ncols - 1);
  ix->setRange(0, iColMax);
  ix->setValue(ad->ix);
  iy->setRange(0, iColMax);
  iy->setValue(ad->iy);
  ie->setRange(0, iColMax);
  ie->setValue(ad->ie);
  iStart->setRange(0, max(0, ad->nrows - 1));
  iStart->setValue(ad->istart);
  n->setRange(0, ad->nrows);
  n->setValue(ad->n);
}

void ArrayDlg::getValues(bool ok)
{
  if (!getVal(this, efx, &ad->fx, i18n("invalid fx!")))
    return;
  if (!getVal(this, efy, &ad->fy, i18n("invalid fy!")))
    return;
  ad->color = colData->color().rgb();
  ad->ix = ix->value();
  ad->iy = iy->value();
  ad->ie = ie->value();
  ad->symb = symb->value();
  ad->istart = iStart->value();
  ad->n = n->value();
  strcpy(ad->path, fileName->text());
  ad->errbars = err->isChecked();
  if (adt) {
    kp->freeXCond(ad);
    ad->x = adt->x;
    ad->ncols = adt->ncols;
    ad->nrows = adt->nrows;
    free(adt);
    adt = 0;
  }
  ad->istart = max(ad->istart, 0);
  ad->n = min(ad->n, ad->nrows - ad->istart);
  kp->changed = true;
  kp->updWin();
  if (ok)
    accept();
  else
    setEditArrayControls(ad);
}

void ArrayDlg::slotSymbol()
{
  SymbolDlg dlg(this, symb);
  dlg.exec();
}

void ArrayDlg::slotFile()
{
  QString s = KFileDialog::getOpenFileName(0, "*.dat");
  if (!s.isEmpty()){
    if (strcmp(s, fileName->text())) {
      adt = (PARRAYDATA) calloc(1, sizeof(ARRAYDATA));
      if (kp->readFile((char *) ((const char *) s), &adt->x, &adt->ncols, &adt->nrows)) {
        adt->ix = min(ad->ix, adt->ncols - 1);
        adt->iy = min(ad->iy, adt->ncols - 1);
        adt->ie = min(ad->ie, adt->ncols - 1);
        adt->istart = min(ad->istart, adt->nrows - 1);
        adt->n = adt->nrows - adt->istart;
        if (ad->n)
          adt->n = min(ad->n, adt->n);
        setEditArrayControls(adt);
        fileName->setText(s);
      } else {
        free(adt);
        adt = 0;
        KMsgBox::message(this, i18n("Error"), i18n("while loading the file"), 2);
      }
    }
  }
}

void ArrayDlg::slotApply()
{
  getValues(false);
}

void ArrayDlg::slotCancel()
{
  if (adt) {
    kp->freeX(&adt->x);
    free(adt);
  }
  reject();
}

void ArrayDlg::slotHelp()
{
  kapp->invokeHTMLHelp("kpl/index-5.html", "sec-array");
}

void ArrayDlg::slotOK()
{
  getValues(true);
}

