/****************************************************************/
/*                                                              */
/*      autodlg.cpp                                             */
/*                                                              */
/*      This file is a part of kpl - a program for graphical    */
/*      presentation of data sets and functions.                */
/*                                                              */
/*      Copyright (C) 1999 Werner Stille                        */
/*      <stille@uni-freiburg.de>                                */
/*                                                              */
/*      Released under the GPL; see file LICENSE for details.   */
/*                                                              */
/****************************************************************/

#include "autodlg.h"
#include "edittextdlg.h"
#include "symboldlg.h"
#include "utils.h"
#include <kapp.h>
#include <kmsgbox.h>
#include <qlineedit.h>
#include <qcheckbox.h>
#include <qspinbox.h>
#include <qradiobutton.h>
#include <qbuttongroup.h>
#include <qcheckbox.h>

AutoDlg::AutoDlg(QWidget *parent) : QDialog(parent, 0, TRUE)
{
  setCaption(i18n("Autoplot settings"));
  g1 = new QGroupBox(i18n("Window"), this);
  g1->setGeometry(10, 8, 346, 102);
  l1 = new QLabel(i18n("Left margin"), this);
  l1->setGeometry(20, 36, 86, 22);
  autoX0 = new QLineEdit(this);
  autoX0->setGeometry(108, 36, 50, 22);
  l2 = new QLabel("cm", this);
  l2->setGeometry(166, 36, 24, 22);
  l3 = new QLabel(i18n("Bottom margin"), this);
  l3->setGeometry(20, 70, 86, 22);
  autoY0 = new QLineEdit(this);
  autoY0->setGeometry(108, 70, 50, 22);
  l4 = new QLabel("cm", this);
  l4->setGeometry(166, 70, 24, 22);
  l5 = new QLabel(i18n("Width"), this);
  l5->setGeometry(208, 36, 54, 22);
  autoXl = new QLineEdit(this);
  autoXl->setGeometry(262, 36, 50, 22);
  l6 = new QLabel("cm", this);
  l6->setGeometry(320, 36, 24, 22);
  l7 = new QLabel(i18n("Height"), this);
  l7->setGeometry(208, 70, 54, 22);
  autoYl = new QLineEdit(this);
  autoYl->setGeometry(262, 70, 50, 22);
  l8 = new QLabel("cm", this);
  l8->setGeometry(320, 70, 24, 22);
  g2 = new QGroupBox(i18n("Labels"), this);
  g2->setGeometry(10, 118, 346, 136);
  l9 = new QLabel(i18n("x axis"), this);
  l9->setGeometry(20, 147, 64, 22);
  autoLetX = new QLineEdit(this);
  autoLetX->setGeometry(84, 147, 226, 22);
  editLetX = new QPushButton("...", this);
  editLetX->setGeometry(320, 146, 25, 25);
  connect(editLetX, SIGNAL(clicked()), SLOT(slotEditLetX()));
  l10 = new QLabel(i18n("y axis"), this);
  l10->setGeometry(20, 181, 64, 22);
  autoLetY = new QLineEdit(this);
  autoLetY->setGeometry(84, 181, 226, 22);
  editLetY = new QPushButton("...", this);
  editLetY->setGeometry(320, 180, 25, 25);
  connect(editLetY, SIGNAL(clicked()), SLOT(slotEditLetY()));
  l11 = new QLabel(i18n("Heading"), this);
  l11->setGeometry(20, 215, 60, 22);
  autoLetH = new QLineEdit(this);
  autoLetH->setGeometry(140, 215, 170, 22);
  editLetH = new QPushButton("...", this);
  editLetH->setGeometry(320, 214, 25, 25);
  connect(editLetH, SIGNAL(clicked()), SLOT(slotEditLetH()));
  autohPath = new QCheckBox(i18n("= path"), this);
  autohPath->setGeometry(80, 215, 58, 22);
  bg = new QButtonGroup(i18n("Axis mode"), this);
  bg->setGeometry(370, 8, 194, 246);
  autoFrameOnly = new QRadioButton(i18n("Frame only"), bg);
  autoFrameOnly->setGeometry(16, 38, 175, 24);
  autoTics = new QRadioButton(i18n("Tics"), bg);
  autoTics->setGeometry(16, 80, 175, 24);
  autoTicsLet = new QRadioButton(i18n("Tics with numbers"), bg);
  autoTicsLet->setGeometry(16, 122, 175, 24);
  autoGrid = new QRadioButton(i18n("Grid"), bg);
  autoGrid->setGeometry(16, 164, 175, 24);
  autoGridLet = new QRadioButton(i18n("Grid with numbers"), bg);
  autoGridLet->setGeometry(16, 206, 175, 24);
  g3 = new QGroupBox(i18n("Data"), this);
  g3->setGeometry(10, 260, 554, 98);
  l12 = new QLabel(i18n("x column"), this);
  l12->setGeometry(20, 288, 64, 22);
  ix = new QSpinBox(0, 19, 1, this);
  ix->setGeometry(84, 288, 58, 22);
  l13 = new QLabel(i18n("y column"), this);
  l13->setGeometry(156, 288, 64, 22);
  iy = new QSpinBox(0, 19, 1, this);
  iy->setGeometry(220, 288, 58, 22);
  l14 = new QLabel(i18n("Error column"), this);
  l14->setGeometry(286, 288, 90, 22);
  ie = new QSpinBox(0, 19, 1, this);
  ie->setGeometry(376, 288, 58, 22);
  err = new QCheckBox(i18n("Error bar"), this);
  err->setGeometry(442, 288, 112, 22);
  l15 = new QLabel(i18n("Symbol"), this);
  l15->setGeometry(20, 322, 64, 22);
  autoSymb = new QSpinBox(-17, 9, 1, this);
  autoSymb->setGeometry(84, 322, 58, 22);
  chsymb = new QPushButton("...", this);
  chsymb->setGeometry(152, 321, 25, 25);
  connect(chsymb, SIGNAL(clicked()), SLOT(slotSymbol()));
  autoNorm = new QCheckBox(i18n("Normalization"), this);
  autoNorm->setGeometry(442, 322, 112, 22);
  ok = new QPushButton(i18n("OK"), this);
  ok->setGeometry(10, 366, 100, 25);
  ok->setDefault(true);
  connect(ok, SIGNAL(clicked()), SLOT(slotOK()));
  cancel = new QPushButton(i18n("Cancel"), this);
  cancel->setGeometry(120, 366, 100, 25);
  connect(cancel, SIGNAL(clicked()), SLOT(reject()));
  help = new QPushButton(i18n("Help"), this);
  help->setGeometry(464, 366, 100, 25);
  connect(help, SIGNAL(clicked()), SLOT(slotHelp()));
  adjustSize();
  setFixedSize(size());
}

AutoDlg::~AutoDlg()
{
  delete g1;
  delete g2;
  delete g3;
  delete l1;
  delete l2;
  delete l3;
  delete l4;
  delete l5;
  delete l6;
  delete l7;
  delete l8;
  delete l9;
  delete l10;
  delete l11;
  delete l12;
  delete l13;
  delete l14;
  delete l15;
  delete autoX0;
  delete autoY0;
  delete autoXl;
  delete autoYl;
  delete autoLetX;
  delete autoLetY;
  delete autoLetH;
  delete editLetX;
  delete editLetY;
  delete editLetH;
  delete chsymb;
  delete autohPath;
  delete err;
  delete autoNorm;
  delete autoFrameOnly;
  delete autoTics;
  delete autoTicsLet;
  delete autoGrid;
  delete autoGridLet;
  delete bg;
  delete ok;
  delete cancel;
  delete help;
}

void AutoDlg::setWidgets(autoStruct as)
{
  QString s;
  QRadioButton *rb;

  autoX0->setText(s.setNum(as.x0Auto, 'f', 2));
  autoXl->setText(s.setNum(as.xlAuto, 'f', 2));
  autoY0->setText(s.setNum(as.y0Auto, 'f', 2));
  autoYl->setText(s.setNum(as.ylAuto, 'f', 2));
  autoLetX->setText(as.sxAuto);
  autoLetY->setText(as.syAuto);
  autoLetH->setText(as.shAuto);
  autohPath->setChecked(as.autohPath);
  rb = (QRadioButton *) bg->find(as.autoGrid);
  rb->setChecked(true);
  ix->setValue(as.ixAuto);
  iy->setValue(as.iyAuto);
  ie->setValue(as.ieAuto);
  err->setChecked(as.autoErr);
  autoSymb->setValue(as.autoSymb);
  autoNorm->setChecked(as.autoNorm);
}

void AutoDlg::editText(QLineEdit *ed)
{
  EditTextDlg dlg(this, ed);
  dlg.exec();
}

void AutoDlg::slotEditLetX()
{
  editText(autoLetX);
}

void AutoDlg::slotEditLetY()
{
  editText(autoLetY);
}

void AutoDlg::slotEditLetH()
{
  editText(autoLetH);
}

void AutoDlg::slotSymbol()
{
  SymbolDlg dlg(this, autoSymb);
  dlg.exec();
}

void AutoDlg::slotHelp()
{
  kapp->invokeHTMLHelp("kpl/index-5.html", "sec-autoplot");
}

autoStruct AutoDlg::getValues()
{
  QRadioButton *rb;

  strcpy(a.sxAuto, autoLetX->text());
  strcpy(a.syAuto, autoLetY->text());
  strcpy(a.shAuto, autoLetH->text());
  a.autohPath = autohPath->isChecked();
  a.autoGrid = -1;
  do 
    rb = (QRadioButton *) bg->find(++a.autoGrid);
  while (!rb->isChecked());
  a.ixAuto = ix->value();
  a.iyAuto = iy->value();
  a.ieAuto = ie->value();
  a.autoErr = err->isChecked();
  a.autoSymb = autoSymb->value();
  a.autoNorm = autoNorm->isChecked();
  return a;
}

void AutoDlg::slotOK()
{
  if (!getVal(this, autoX0, &a.x0Auto, i18n("invalid position of x!")))
    return;
  if (!getVal(this, autoY0, &a.y0Auto, i18n("invalid position of y!")))
    return;
  if (!getVal(this, autoXl, &a.xlAuto, i18n("invalid width!")))
    return;
  if (!getVal(this, autoYl, &a.ylAuto, i18n("invalid height!")))
    return;
  accept();
}

