/****************************************************************/
/*                                                              */
/*      edittextdlg.cpp                                         */
/*                                                              */
/*      This file is a part of kpl - a program for graphical    */
/*      presentation of data sets and functions.                */
/*                                                              */
/*      Copyright (C) 1999 Werner Stille                        */
/*      <stille@uni-freiburg.de>                                */
/*                                                              */
/*      Released under the GPL; see file LICENSE for details.   */
/*                                                              */
/****************************************************************/

#include "edittextdlg.h"
#include <kapp.h>
#include <qpushbutton.h>
#include <qlineedit.h>
#include <qbuttongroup.h> 
#include <qradiobutton.h>

Preview::Preview(QWidget *parent) : QWidget(parent)
{
  QWidget::setBackgroundColor(white);
  str[0] = 0;
}

void Preview::paintEvent(QPaintEvent *)
{
  plInit(this, 0, 0);
  relsize(2.0);
  text(str, 10, 20, 1);
  end();
}

void Preview::setText(const char *s)
{
  strcpy(str, s);
  update();
}

EditTextDlg::EditTextDlg(QWidget *parent, QLineEdit *e) :
 QDialog(parent, 0, TRUE)
{
  ed = e;
  setCaption(i18n("Edit text"));
  strcpy(sfont, "#NN");
  editPreview = new Preview(this);
  editPreview->setGeometry(8, 12, 276, 40);
  editPreview->setText(ed->text());
  text = new QLineEdit(this);
  text->setGeometry(8, 62, 174, 24);
  text->setText(ed->text());
  text->setCursorPosition(strlen(ed->text()));
  connect(text, SIGNAL(textChanged(const char *)),
          editPreview, SLOT(setText(const char *)));
  font = new QLineEdit(this);
  font->setGeometry(228, 62, 56, 24);
  font->setText(sfont);
  cat = new QPushButton("<", this);
  cat->setGeometry(194, 62, 25, 25);
  connect(cat, SIGNAL(clicked()), SLOT(slotCat()));
  bg1 = new QButtonGroup(i18n("Font"), this);
  bg1->setGeometry(8, 92, 174, 176);
  helv = new QRadioButton("Helvetica", bg1);
  helv->setGeometry(18, 20, 150, 24);
  connect(helv, SIGNAL(clicked()), SLOT(slotFont()));
  helvB = new QRadioButton("Helvetica Bold", bg1);
  helvB->setGeometry(18, 44, 150, 24);
  connect(helvB, SIGNAL(clicked()), SLOT(slotFont()));
  helvI = new QRadioButton("Helvetica Italic", bg1);
  helvI->setGeometry(18, 68, 150, 24);
  connect(helvI, SIGNAL(clicked()), SLOT(slotFont()));
  helvBI = new QRadioButton("Helvetica Bold Italic", bg1);
  helvBI->setGeometry(18, 92, 150, 24);
  connect(helvBI, SIGNAL(clicked()), SLOT(slotFont()));
  symb = new QRadioButton("Symbol", bg1);
  symb->setGeometry(18, 116, 150, 24);
  connect(symb, SIGNAL(clicked()), SLOT(slotFont()));
  zapf = new QRadioButton("ZapfDingbats", bg1);
  zapf->setGeometry(18, 140, 150, 24);
  connect(zapf, SIGNAL(clicked()), SLOT(slotFont()));
  helv->setChecked(true);
  bg2 = new QButtonGroup(i18n("Position"), this);
  bg2->setGeometry(194, 92, 90, 98);
  up = new QRadioButton(i18n("Up"), bg2);
  up->setGeometry(12, 20, 75, 24);
  connect(up, SIGNAL(clicked()), SLOT(slotPos()));
  normal = new QRadioButton(i18n("Normal"), bg2);
  normal->setGeometry(12, 44, 75, 24);
  connect(normal, SIGNAL(clicked()), SLOT(slotPos()));
  down = new QRadioButton(i18n("Down"), bg2);
  down->setGeometry(12, 68, 75, 24);
  connect(down, SIGNAL(clicked()), SLOT(slotPos()));
  normal->setChecked(true);
  ok = new QPushButton(i18n("OK"), this);
  ok->setGeometry(194, 202, 90, 25);
  ok->setDefault(true);
  connect(ok, SIGNAL(clicked()), SLOT(slotOK()));
  cancel = new QPushButton(i18n("Cancel"), this);
  cancel->setGeometry(194, 240, 90, 25);
  connect(cancel, SIGNAL(clicked()), SLOT(reject()));
  adjustSize();
  setFixedSize(size());
}

EditTextDlg::~EditTextDlg()
{
  delete text;
  delete font;
  delete editPreview;
  delete helv;
  delete helvB;
  delete helvI;
  delete helvBI;
  delete symb;
  delete zapf;
  delete up;
  delete normal;
  delete down;
  delete bg1;
  delete bg2;
  delete cat;
  delete ok;
  delete cancel;
}

void EditTextDlg::slotCat()
{
  text->insert(font->text());
}

void EditTextDlg::slotFont()
{
  int i = -1;
  char fs[] = "HBOISZ";
  QRadioButton *rb;

  do 
    rb = (QRadioButton *) bg1->find(++i);
  while (!rb->isChecked());
  sfont[1] = fs[i];
  font->setText(sfont);
}

void EditTextDlg::slotPos()
{
  int i = -1;
  char ps[] = "UND";
  QRadioButton *rb;

  do 
    rb = (QRadioButton *) bg2->find(++i);
  while (!rb->isChecked());
  sfont[2] = ps[i];
  font->setText(sfont);
}

void EditTextDlg::slotOK()
{
  ed->setText(text->text());
  accept();
}

