/****************************************************************/
/*                                                              */
/*      framedlg.cpp                                            */
/*                                                              */
/*      This file is a part of kpl - a program for graphical    */
/*      presentation of data sets and functions.                */
/*                                                              */
/*      Copyright (C) 1999 Werner Stille                        */
/*      <stille@uni-freiburg.de>                                */
/*                                                              */
/*      Released under the GPL; see file LICENSE for details.   */
/*                                                              */
/****************************************************************/

#include "kpl.h"
#include "framedlg.h"
#include "edittextdlg.h"
#include "utils.h"
#include <kapp.h>
#include <kmsgbox.h>
#include <kcolorbtn.h>
#include <qcheckbox.h>
#include <qspinbox.h>
#include <qbuttongroup.h> 
#include <qradiobutton.h>
#include <qlineedit.h>
#include <math.h>

#ifndef min
  #define min(x,y) ((x) < (y) ? (x) : (y))
#endif
#ifndef max
  #define max(x,y) ((x) > (y) ? (x) : (y))
#endif

FrameDlg::FrameDlg(QWidget *parent, KPl *kpa, int it) :
 QDialog(parent, 0, TRUE)
{
  QString s;
  QRadioButton *rb;

  kp = kpa;
  item = it;
  pfd = (PFRAMEDATA) kp->itd[item].stru;
  fd = *pfd;
  iex = iey = 999;
  setCaption(i18n("Frame"));
  bg = new QButtonGroup(i18n("Axis mode"), this);
  bg->setGeometry(10, 6, 190, 198);
  FrameOnly = new QRadioButton(i18n("Frame only"), bg);
  FrameOnly->setGeometry(12, 32, 175, 24);
  Tics = new QRadioButton(i18n("Tics"), bg);
  Tics->setGeometry(12, 64, 175, 24);
  TicsLet = new QRadioButton(i18n("Tics with numbers"), bg);
  TicsLet->setGeometry(12, 96, 175, 24);
  Grid = new QRadioButton(i18n("Grid"), bg);
  Grid->setGeometry(12, 128, 175, 24);
  GridLet = new QRadioButton(i18n("Grid with numbers"), bg);
  GridLet->setGeometry(12, 160, 175, 24);
  rb = (QRadioButton *) bg->find(fd.gridmode);
  rb->setChecked(true);
  g1 = new QGroupBox(i18n("Window"), this);
  g1->setGeometry(210, 6, 318, 98);
  l1 = new QLabel(i18n("Left margin"), this);
  l1->setGeometry(220, 30, 84, 22);
  ex0 = new QLineEdit(this);
  ex0->setGeometry(304, 30, 50, 22);
  ex0->setText(s.setNum(fd.xw0, 'f', 2));
  l2 = new QLabel("cm", this);
  l2->setGeometry(360, 30, 30, 22);
  l3 = new QLabel(i18n("Width"), this);
  l3->setGeometry(396, 30, 48, 22);
  exl = new QLineEdit(this);
  exl->setGeometry(444, 30, 50, 22);
  exl->setText(s.setNum(fd.xw, 'f', 2));
  l4 = new QLabel("cm", this);
  l4->setGeometry(500, 30, 25, 22);
  l5 = new QLabel(i18n("Bottom margin"), this);
  l5->setGeometry(220, 66, 84, 22);
  ey0 = new QLineEdit(this);
  ey0->setGeometry(304, 66, 50, 22);
  ey0->setText(s.setNum(fd.yw0, 'f', 2));
  l6 = new QLabel("cm", this);
  l6->setGeometry(360, 66, 30, 22);
  l7 = new QLabel(i18n("Height"), this);
  l7->setGeometry(396, 66, 48, 22);
  eyl = new QLineEdit(this);
  eyl->setGeometry(444, 66, 50, 22);
  eyl->setText(s.setNum(fd.yw, 'f', 2));
  l8 = new QLabel("cm", this);
  l8->setGeometry(500, 66, 25, 22);
  g2 = new QGroupBox(i18n("Scaling"), this);
  g2->setGeometry(210, 110, 318, 94);
  l9 = new QLabel("xmin", this);
  l9->setGeometry(220, 134, 40, 22);
  exMin = new QLineEdit(this);
  exMin->setGeometry(260, 134, 80, 22);
  exMin->setText(s.setNum(fd.xmi));
  l10 = new QLabel("xmax", this);
  l10->setGeometry(374, 134, 40, 22);
  exMax = new QLineEdit(this);
  exMax->setGeometry(414, 134, 80, 22);
  exMax->setText(s.setNum(fd.xma));
  l11 = new QLabel("ymin", this);
  l11->setGeometry(220, 168, 40, 22);
  eyMin = new QLineEdit(this);
  eyMin->setGeometry(260, 168, 80, 22);
  eyMin->setText(s.setNum(fd.ymi));
  l12 = new QLabel("ymax", this);
  l12->setGeometry(372, 168, 40, 22);
  eyMax = new QLineEdit(this);
  eyMax->setGeometry(414, 168, 80, 22);
  eyMax->setText(s.setNum(fd.yma));
  g3 = new QGroupBox(i18n("Colors"), this);
  g3->setGeometry(10, 210, 190, 88);
  l13 = new QLabel(i18n("Frame"), this);
  l13->setGeometry(20, 233, 68, 25);
  colFrame = new KColorButton(this, 0);
  colFrame->setGeometry(88, 233, 80, 25);
  colFrame->setColor(QColor(fd.colFrame));
  l14 = new QLabel(i18n("Grid"), this);
  l14->setGeometry(20, 265, 68, 25);
  colGrid = new KColorButton(this, 0);
  colGrid->setGeometry(88, 265, 80, 25);
  colGrid->setColor(QColor(fd.colGrid));
  g4 = new QGroupBox(i18n("Axes"), this);
  g4->setGeometry(210, 210, 318, 88);
  l15 = new QLabel("xtic", this);
  l15->setGeometry(220, 232, 42, 22);
  exTic = new QLineEdit(this);
  exTic->setGeometry(262, 232, 50, 22);
  exTic->setText(s.setNum(fd.xtic));
  l16 = new QLabel("mticx", this);
  l16->setGeometry(328, 232, 40, 22);
  emTicx = new QSpinBox(0, 10, 1, this);
  emTicx->setGeometry(368, 232, 44, 22);
  emTicx->setValue(fd.mticx);
  l17 = new QLabel("ndigx", this);
  l17->setGeometry(426, 232, 44, 22);
  enDigx = new QSpinBox(-1, 8, 1, this);
  enDigx->setGeometry(470, 232, 44, 22);
  enDigx->setValue(fd.ndigx);
  l18 = new QLabel("ytic", this);
  l18->setGeometry(220, 266, 42, 22);
  eyTic = new QLineEdit(this);
  eyTic->setGeometry(262, 266, 50, 22);
  eyTic->setText(s.setNum(fd.ytic));
  l19 = new QLabel("mticy", this);
  l19->setGeometry(328, 266, 40, 22);
  emTicy = new QSpinBox(0, 10, 1, this);
  emTicy->setGeometry(368, 266, 44, 22);
  emTicy->setValue(fd.mticy);
  l20 = new QLabel("ndigy", this);
  l20->setGeometry(426, 266, 44, 22);
  enDigy = new QSpinBox(-1, 8, 1, this);
  enDigy->setGeometry(470, 266, 44, 22);
  enDigy->setValue(fd.ndigy);
  g5 = new QGroupBox(i18n("Automatic"), this);
  g5->setGeometry(10, 304, 190, 88);
  autoNorm = new QCheckBox(i18n("Normalization"), this);
  autoNorm->setGeometry(20, 326, 170, 22);
  autoNorm->setChecked(kp->aut.autoNorm);
  autoScale = new QPushButton(i18n("Autoscale"), this);
  autoScale->setGeometry(20, 354, 166, 25);
  connect(autoScale, SIGNAL(clicked()), SLOT(slotAutoScale()));
  g6 = new QGroupBox(i18n("Labels"), this);
  g6->setGeometry(210, 304, 318, 124);
  l21 = new QLabel("iex", this);
  l21->setGeometry(220, 326, 26, 22);
  ex = new QSpinBox(-99, 99, 1, this);
  ex->setGeometry(262, 326, 50, 22);
  ex->setValue(fd.iex);
  l22 = new QLabel(i18n("x axis"), this);
  l22->setGeometry(328, 326, 62, 22);
  eLetX = new QLineEdit(this);
  eLetX->setGeometry(390, 326, 90, 22);
  eLetX->setText(fd.sx);
  editLetX = new QPushButton("...", this);
  editLetX->setGeometry(490, 325, 25, 25);
  connect(editLetX, SIGNAL(clicked()), SLOT(slotEditLetX()));
  l23 = new QLabel("iey", this);
  l23->setGeometry(220, 360, 28, 22);
  ey = new QSpinBox(-99, 99, 1, this);
  ey->setGeometry(262, 360, 50, 22);
  ey->setValue(fd.iey);
  l24 = new QLabel(i18n("y axis"), this);
  l24->setGeometry(328, 360, 62, 22);
  eLetY = new QLineEdit(this);
  eLetY->setGeometry(390, 360, 90, 22);
  eLetY->setText(fd.sy);
  editLetY = new QPushButton("...", this);
  editLetY->setGeometry(490, 359, 25, 25);
  connect(editLetY, SIGNAL(clicked()), SLOT(slotEditLetY()));
  l25 = new QLabel(i18n("Heading"), this);
  l25->setGeometry(328, 394, 62, 22);
  eLetH = new QLineEdit(this);
  eLetH->setGeometry(390, 394, 90, 22);
  eLetH->setText(fd.sh);
  editLetH = new QPushButton("...", this);
  editLetH->setGeometry(490, 393, 25, 25);
  connect(editLetH, SIGNAL(clicked()), SLOT(slotEditLetH()));
  l26 = new QLabel(i18n("Symbol size"), this);
  l26->setGeometry(12, 404, 98, 22);
  eRelSiz = new QSpinBox(10, 800, 1, this);
  eRelSiz->setGeometry(114, 404, 54, 22);
  eRelSiz->setValue((int) (100 * fd.relsiz + 0.5));
  l27 = new QLabel("%", this);
  l27->setGeometry(176, 404, 30, 22);
  ok = new QPushButton(i18n("OK"), this);
  ok->setGeometry(12, 442, 100, 25);
  ok->setDefault(true);
  apply = new QPushButton(i18n("Apply"), this);
  apply->setGeometry(122, 442, 100, 25);
  connect(apply, SIGNAL(clicked()), SLOT(slotApply()));
  connect(ok, SIGNAL(clicked()), SLOT(slotOK()));
  cancel = new QPushButton(i18n("Cancel"), this);
  cancel->setGeometry(232, 442, 100, 25);
  connect(cancel, SIGNAL(clicked()), SLOT(reject()));
  help = new QPushButton(i18n("Help"), this);
  help->setGeometry(428, 442, 100, 25);
  connect(help, SIGNAL(clicked()), SLOT(slotHelp()));
  adjustSize();
  setFixedSize(size());
}

FrameDlg::~FrameDlg()
{
  delete g1;
  delete g2;
  delete g3;
  delete g4;
  delete g5;
  delete g6;
  delete l1;
  delete l2;
  delete l3;
  delete l4;
  delete l5;
  delete l6;
  delete l7;
  delete l8;
  delete l9;
  delete l10;
  delete l11;
  delete l12;
  delete l13;
  delete l14;
  delete l15;
  delete l16;
  delete l17;
  delete l18;
  delete l19;
  delete l20;
  delete l21;
  delete l22;
  delete l23;
  delete l24;
  delete l25;
  delete l26;
  delete l27;
  delete ex0;
  delete ey0;
  delete exl;
  delete eyl;
  delete exMin;
  delete exMax;
  delete eyMin;
  delete eyMax;
  delete exTic;
  delete eyTic;
  delete eLetX;
  delete eLetY;
  delete eLetH;
  delete editLetX;
  delete editLetY;
  delete editLetH;
  delete autoNorm;
  delete FrameOnly;
  delete Tics;
  delete TicsLet;
  delete Grid;
  delete GridLet;
  delete colFrame;
  delete colGrid;
  delete emTicx;
  delete emTicy;
  delete enDigx;
  delete enDigy;
  delete ex;
  delete ey;
  delete eRelSiz;
  delete bg;
  delete apply;
  delete ok;
  delete cancel;
  delete help;
}

void FrameDlg::editText(QLineEdit *ed)
{
  EditTextDlg dlg(this, ed);
  dlg.exec();
}

void FrameDlg::slotEditLetX()
{
  editText(eLetX);
}

void FrameDlg::slotEditLetY()
{
  editText(eLetY);
}

void FrameDlg::slotEditLetH()
{
  editText(eLetH);
}

void FrameDlg::slotAutoScale()
{
  int i, iext, ieyt, mtic, ndig;
  double xmi, xma, ymi, yma, xtic, ytic, xmin, xmax, ymin, ymax,
         xl, yl, fxt, fyt;
  QString s;

  if (autoNorm->isChecked()) {
    iex = iey = 999;
    for (i = item + 1; i < kp->nItems; i++) {
      switch (kp->itd[i].ityp) {
        case ARRAYITEM:
          kp->expoFl(&iext, &fxt,
                     &((PARRAYDATA) kp->itd[i].stru)->x
                     [((PARRAYDATA) kp->itd[i].stru)->ix]
                     [((PARRAYDATA) kp->itd[i].stru)->istart],
                     ((PARRAYDATA) kp->itd[i].stru)->n);
          kp->expoFl(&ieyt, &fyt,
                     &((PARRAYDATA) kp->itd[i].stru)->x
                     [((PARRAYDATA) kp->itd[i].stru)->iy]
                     [((PARRAYDATA) kp->itd[i].stru)->istart],
                     ((PARRAYDATA) kp->itd[i].stru)->n);
          break;
        case FUNITEM:
          kp->expo(max(fabs(((PFUNDATA) kp->itd[i].stru)->xfmin),
                   fabs(((PFUNDATA) kp->itd[i].stru)->xfmax)),
                   &iext, &fxt);
          kp->expoFn(((PFUNDATA) kp->itd[i].stru)->fkt, &ieyt, &fyt,
                     ((PFUNDATA) kp->itd[i].stru)->xfmin,
                     ((PFUNDATA) kp->itd[i].stru)->xfmax,
                     ((PFUNDATA) kp->itd[i].stru)->dx,
                     ((PFUNDATA) kp->itd[i].stru)->p);
          break;
        case PARFUNITEM:
          kp->expoFn(((PPARFUNDATA) kp->itd[i].stru)->fktx, &iext, &fxt,
                     ((PPARFUNDATA) kp->itd[i].stru)->tmin,
                     ((PPARFUNDATA) kp->itd[i].stru)->tmax,
                     ((PPARFUNDATA) kp->itd[i].stru)->dt,
                     ((PPARFUNDATA) kp->itd[i].stru)->px);
          kp->expoFn(((PPARFUNDATA) kp->itd[i].stru)->fkty, &ieyt, &fyt,
                     ((PPARFUNDATA) kp->itd[i].stru)->tmin,
                     ((PPARFUNDATA) kp->itd[i].stru)->tmax,
                     ((PPARFUNDATA) kp->itd[i].stru)->dt,
                     ((PPARFUNDATA) kp->itd[i].stru)->py);
      }
      if (kp->itd[i].ityp == FRAMEITEM)
        break;
      if (iext < iex) {
        iex = iext;
        fx = fxt;
      }
      if (ieyt < iey) {
        iey = ieyt;
        fy = fyt;
      }
    }
  } else {
    iex = iey = 0;
    fx = fy = 1.0;
  }
  if ((iex != 999) && (iey != 999)) {
    if (!getVal(this, exl, &xl, i18n("invalid width!")))
      return;
    if (!getVal(this, eyl, &yl, i18n("invalid height!")))
      return;
    xmin = ymin = 1.0e300;
    xmax = ymax = -xmin;
    for (i = item + 1; i < kp->nItems; i++) {
      switch (kp->itd[i].ityp) {
        case ARRAYITEM:
          kp->minMaxFl(&xmi, &xma,
                       &((PARRAYDATA) kp->itd[i].stru)->x
                       [((PARRAYDATA) kp->itd[i].stru)->ix]
                       [((PARRAYDATA) kp->itd[i].stru)->istart],
                       ((PARRAYDATA) kp->itd[i].stru)->n);
          kp->minMaxFl(&ymi, &yma,
                       &((PARRAYDATA) kp->itd[i].stru)->x
                       [((PARRAYDATA) kp->itd[i].stru)->iy]
                       [((PARRAYDATA) kp->itd[i].stru)->istart],
                       ((PARRAYDATA) kp->itd[i].stru)->n);
          break;
        case FUNITEM:
          xmi = ((PFUNDATA) kp->itd[i].stru)->xfmin;
          xma = ((PFUNDATA) kp->itd[i].stru)->xfmax;
          kp->autoFn(&ymi, &yma, &ytic, &mtic, &ndig,
                     ((PFUNDATA) kp->itd[i].stru)->fkt,
                     xmi, xma,
                     ((PFUNDATA) kp->itd[i].stru)->dx,
                     ((PFUNDATA) kp->itd[i].stru)->p,
                     1.0, 0.0);
          break;
        case PARFUNITEM:
          kp->autoFn(&xmi, &xma, &xtic, &mtic, &ndig,
                     ((PPARFUNDATA) kp->itd[i].stru)->fktx,
                     ((PPARFUNDATA) kp->itd[i].stru)->tmin,
                     ((PPARFUNDATA) kp->itd[i].stru)->tmax,
                     ((PPARFUNDATA) kp->itd[i].stru)->dt,
                     ((PPARFUNDATA) kp->itd[i].stru)->px,
                     1.0, 0.0);
          kp->autoFn(&ymi, &yma, &ytic, &mtic, &ndig,
                     ((PPARFUNDATA) kp->itd[i].stru)->fkty,
                     ((PPARFUNDATA) kp->itd[i].stru)->tmin,
                     ((PPARFUNDATA) kp->itd[i].stru)->tmax,
                     ((PPARFUNDATA) kp->itd[i].stru)->dt,
                     ((PPARFUNDATA) kp->itd[i].stru)->py,
                     1.0, 0.0);
      }
      if (kp->itd[i].ityp == FRAMEITEM)
        break;
      xmin = min(xmin, xmi);
      xmax = max(xmax, xma);
      ymin = min(ymin, ymi);
      ymax = max(ymax, yma);
    }
    kp->autoSc(&xmi, &xma, &xtic, &mtic, &ndig, xmin, xmax, fx, 0.2);
    ex->setValue(iex);
    emTicx->setValue(mtic);
    enDigx->setValue(ndig);
    exMin->setText(s.setNum(xmi));
    exMax->setText(s.setNum(xma));
    exTic->setText(s.setNum(xtic));
    kp->autoSc(&ymi, &yma, &ytic, &mtic, &ndig, ymin, ymax, fy,
               0.2 * xl / yl);
    ey->setValue(iey);
    emTicy->setValue(mtic);
    enDigy->setValue(ndig);
    eyMin->setText(s.setNum(ymi));
    eyMax->setText(s.setNum(yma));
    eyTic->setText(s.setNum(ytic));
  }
}

void FrameDlg::getValues(bool ok)
{
  int i;
  QString s;
  QRadioButton *rb;

  if (!getVal(this, ex0, &fd.xw0, i18n("invalid position of x!")))
    return;
  if (!getVal(this, ey0, &fd.yw0, i18n("invalid position of y!")))
    return;
  if (!getVal(this, exl, &fd.xw, i18n("invalid width!")))
    return;
  if (!getVal(this, eyl, &fd.yw, i18n("invalid height!")))
    return;
  if (!getVal(this, exMin, &fd.xmi, i18n("invalid xmin!")))
    return;
  if (!getVal(this, exMax, &fd.xma, i18n("invalid xmax!")))
    return;
  if (!getVal(this, eyMin, &fd.ymi, i18n("invalid ymin!")))
    return;
  if (!getVal(this, eyMax, &fd.yma, i18n("invalid ymax!")))
    return;
  if (!getVal(this, exTic, &fd.xtic, i18n("invalid xtic!")))
    return;
  if (!getVal(this, eyTic, &fd.ytic, i18n("invalid ytic!")))
    return;
  fd.colFrame = colFrame->color().rgb();
  fd.colGrid = colGrid->color().rgb();
  fd.iex = ex->value();
  fd.iey = ey->value();
  fd.mticx = emTicx->value();
  fd.mticy = emTicy->value();
  fd.ndigx = enDigx->value();
  fd.ndigy = enDigy->value();
  strcpy(fd.sx, eLetX->text());
  strcpy(fd.sy, eLetY->text());
  strcpy(fd.sh, eLetH->text());
  fd.relsiz = 0.01 * eRelSiz->value();
  fd.gridmode = -1;
  do 
    rb = (QRadioButton *) bg->find(++fd.gridmode);
  while (!rb->isChecked());
  if ((iex != 999) && (iey != 999)) {
    for (i = item + 1; i < kp->nItems; i++) {
      switch (kp->itd[i].ityp) {
        case ARRAYITEM:
          ((PARRAYDATA) kp->itd[i].stru)->fx = fx;
          ((PARRAYDATA) kp->itd[i].stru)->fy = fy;
          break;
        case FUNITEM:
          ((PFUNDATA) kp->itd[i].stru)->fx = fx;
          ((PFUNDATA) kp->itd[i].stru)->fy = fy;
          break;
        case PARFUNITEM:
          ((PPARFUNDATA) kp->itd[i].stru)->fx = fx;
          ((PPARFUNDATA) kp->itd[i].stru)->fy = fy;
      }
      if (kp->itd[i].ityp == FRAMEITEM)
        break;
    }
  }
  *pfd = fd;
  kp->changed = true;
  kp->updWin();
  if (ok)
    accept();
}

void FrameDlg::slotApply()
{
  getValues(false);
}

void FrameDlg::slotHelp()
{
  kapp->invokeHTMLHelp("kpl/index-5.html", "sec-frame");
}

void FrameDlg::slotOK()
{
  getValues(true);
}

