/****************************************************************/
/*                                                              */
/*      itemdlg.cpp                                             */
/*                                                              */
/*      This file is a part of kpl - a program for graphical    */
/*      presentation of data sets and functions.                */
/*                                                              */
/*      Copyright (C) 1999 Werner Stille                        */
/*      <stille@uni-freiburg.de>                                */
/*                                                              */
/*      Released under the GPL; see file LICENSE for details.   */
/*                                                              */
/****************************************************************/

#include "itemdlg.h"
#include "kpl.h"
#include "framedlg.h"
#include "arraydlg.h"
#include "funcdlg.h"
#include "legenddlg.h"
#include <kapp.h>
#include <qfileinfo.h>
#include <qradiobutton.h>
#include <qpushbutton.h>
#include <qbuttongroup.h>
#include <qlistbox.h>
#include <dlfcn.h>

NewItemDlg::NewItemDlg(QWidget *parent, int *i) :
 QDialog(parent, 0, TRUE)
{
  ityp = i;
  setCaption(i18n("New item"));
  bg = new QButtonGroup();
  newFrame = groupButton(i18n("Frame"), 14, 22, 160, 24);
  newArray = groupButton(i18n("Array"), 14, 54, 160, 24);
  newFun = groupButton(i18n("Function"), 14, 86, 160, 24);
  newParFun = groupButton(i18n("Parametric Function"), 14, 118, 160, 24);
  newLegend = groupButton(i18n("Legend"), 14, 150, 160, 24);
  newFrame->setChecked(true);
  ok = new QPushButton(i18n("OK"), this);
  ok->setGeometry(48, 192, 100, 25);
  ok->setDefault(true);
  connect(ok, SIGNAL(clicked()), SLOT(slotOK()));
  adjustSize();
  setFixedSize(size());
}

NewItemDlg::~NewItemDlg()
{
  delete bg;
  delete newFrame;
  delete newArray;
  delete newFun;
  delete newParFun;
  delete newLegend;
  delete ok;
}

QRadioButton *NewItemDlg::groupButton(const char *text,
                                      int x, int y, int w, int h)
{
  QRadioButton *b;

  b = new QRadioButton(text, this);
  b->setGeometry(x, y, w, h);
  bg->insert(b);
  return b;
}

void NewItemDlg::slotOK()
{
  QRadioButton *rb;

  *ityp = 0;
  do 
    rb = (QRadioButton *) bg->find((*ityp)++);
  while (!rb->isChecked());
  accept();
}

ItemDlg::ItemDlg(QWidget *parent) : QDialog(parent, 0, TRUE)
{
  kp = (KPl *) parent;
  change = false;
  setCaption(i18n("Items"));
  itemList = new QListBox(this);
  itemList->setGeometry(8, 12, 258, 205);
  connect(itemList, SIGNAL(selected(int)), SLOT(slotList(int)));
  newItem = new QPushButton(i18n("New"), this);
  newItem->setGeometry(276, 12, 90, 25);
  connect(newItem, SIGNAL(clicked()), SLOT(slotNew()));
  edit = new QPushButton(i18n("Edit"), this);
  edit->setGeometry(276, 48, 90, 25);
  connect(edit, SIGNAL(clicked()), SLOT(slotEdit()));
  del = new QPushButton(i18n("Delete"), this);
  del->setGeometry(276, 84, 90, 25);
  connect(del, SIGNAL(clicked()), SLOT(slotDelete()));
  copy = new QPushButton(i18n("Copy"), this);
  copy->setGeometry(276, 120, 90, 25);
  connect(copy, SIGNAL(clicked()), SLOT(slotCopy()));
  help = new QPushButton(i18n("Help"), this);
  help->setGeometry(276, 156, 90, 25);
  connect(help, SIGNAL(clicked()), SLOT(slotHelp()));
  ok = new QPushButton(i18n("OK"), this);
  ok->setGeometry(276, 192, 90, 25);
  ok->setDefault(true);
  connect(ok, SIGNAL(clicked()), SLOT(slotOK()));
  adjustSize();
  setFixedSize(size());
  updItemList();
}

ItemDlg::~ItemDlg()
{
  delete itemList;
  delete newItem;
  delete edit;
  delete del;
  delete copy;
  delete help;
  delete ok;
}

/****************************************************************/
/*                                                              */
/*      updItemList()                                           */
/*        updates list of plot items                            */
/*                                                              */
/****************************************************************/

void ItemDlg::updItemList()
{
  int i;
  QString s, s2;
  QFileInfo fi;
  QString fname;

  itemList->clear();
  if (kp->nItems) {
    for (i = 0; i < kp->nItems; i++) {
      switch (kp->itd[i].ityp) {
        case FRAMEITEM:
          s.sprintf("%s %g %g %g %g", i18n("Frame"),
                    ((PFRAMEDATA) kp->itd[i].stru)->xw0,
                    ((PFRAMEDATA) kp->itd[i].stru)->yw0,
                    ((PFRAMEDATA) kp->itd[i].stru)->xw,
                    ((PFRAMEDATA) kp->itd[i].stru)->yw);
          break;
        case ARRAYITEM:
          fi.setFile(((PARRAYDATA) kp->itd[i].stru)->path);
          fname = fi.fileName();
          s.sprintf("%s %s %2i %2i", i18n("Array"), (const char *) fname,
                    ((PARRAYDATA) kp->itd[i].stru)->ix,
                    ((PARRAYDATA) kp->itd[i].stru)->iy);
          break;
        case FUNITEM:
          fi.setFile(((PFUNDATA) kp->itd[i].stru)->path);
          fname = fi.baseName();
          s.sprintf("%s %s.%s", i18n("Function"), (const char *) fname,
                    ((PFUNDATA) kp->itd[i].stru)->name);
          break;
        case PARFUNITEM:
          fi.setFile(((PPARFUNDATA) kp->itd[i].stru)->pathx);
          fname = fi.baseName();
          s2.sprintf("%s %s.%s", i18n("Function"), (const char *) fname,
                     ((PPARFUNDATA) kp->itd[i].stru)->namex);
          fi.setFile(((PPARFUNDATA) kp->itd[i].stru)->pathy);
          fname = fi.baseName();
          s.sprintf("%s %s.%s", (const char *) s2, (const char *) fname,
                    ((PPARFUNDATA) kp->itd[i].stru)->namey);
          break;
        case LEGENDITEM:
          s.sprintf("%s %2i", i18n("Legend"),
                    ((PLEGENDDATA) kp->itd[i].stru)->symb);
      }
      itemList->insertItem(s);
    }
    itemList->setCurrentItem(0);
  }
  del->setEnabled(kp->nItems);
  edit->setEnabled(kp->nItems);
  copy->setEnabled(kp->nItems);
}

/****************************************************************/
/*                                                              */
/*      editItem(i)                                             */
/*        shows dialog to change the settings                   */
/*        of plot items                                         */
/*                                                              */
/*      i:     index of item structure                          */
/*                                                              */
/****************************************************************/

int ItemDlg::editItem(int i)
{
  switch (kp->itd[i].ityp) {
    case FRAMEITEM:
      {
        FrameDlg dlg(this, kp, i);
        return dlg.exec();
      }
    case ARRAYITEM:
      {
        ArrayDlg dlg(this, kp, (PARRAYDATA) kp->itd[i].stru);
        return dlg.exec();
      }
    case FUNITEM:
      {
        FuncDlg dlg(this, kp, (PFUNDATA) kp->itd[i].stru);
        return dlg.exec();
      }
    case PARFUNITEM:
      {
        ParFuncDlg dlg(this, kp, (PPARFUNDATA) kp->itd[i].stru);
        return dlg.exec();
      }
    case LEGENDITEM:
      {
        LegendDlg dlg(this, kp, (PLEGENDDATA) kp->itd[i].stru);
        return dlg.exec();
      }
  }
  return 0;
}

void ItemDlg::slotNew()
{
  int i;

  (kp->nItems)++;
  kp->itd = (PITEMDATA) realloc(kp->itd, kp->nItems * sizeof(ITEMDATA));
  NewItemDlg dlg(this, &i);
  dlg.exec();
  kp->iniStru(&(kp->itd[kp->nItems - 1]), i);
  editItem(kp->nItems - 1);
  updItemList();
  change = true;
}

void ItemDlg::slotList(int i)
{
  if (editItem(i))
    change = true;
  updItemList();
}

void ItemDlg::slotEdit()
{
  int i;

  if ((i = itemList->currentItem()) != -1)
    slotList(i);
}

void ItemDlg::slotDelete()
{
  int i,j;

  if ((i = itemList->currentItem()) != -1) {
    kp->freeItd(&(kp->itd[i]));
    kp->nItems--;
    for (j = i; j < kp->nItems; j++)
      kp->itd[j] = kp->itd[j + 1];
      updItemList();
      change = true;
  }
}

/****************************************************************/
/*                                                              */
/*      copyStru(id, is)                                        */
/*        copies plot item structure                            */
/*                                                              */
/*      id:    target index                                     */
/*      is:    source index                                     */
/*                                                              */
/****************************************************************/

void ItemDlg::copyStru(int id, int is)
{
  kp->itd[id] = kp->itd[is];
  kp->itd[id].stru = malloc(kp->iSize[kp->itd[id].ityp]);
  memcpy(kp->itd[id].stru, kp->itd[is].stru, kp->iSize[kp->itd[is].ityp]);
  switch (kp->itd[id].ityp) {
    case FUNITEM:
      if (((PFUNDATA) kp->itd[id].stru)->hmod)
        ((PFUNDATA) kp->itd[id].stru)->hmod =
          dlopen(((PFUNDATA) kp->itd[id].stru)->path, RTLD_NOW);
      break;
    case PARFUNITEM:
      if (((PPARFUNDATA) kp->itd[id].stru)->hmodx)
        ((PPARFUNDATA) kp->itd[id].stru)->hmodx =
          dlopen(((PPARFUNDATA) kp->itd[id].stru)->pathx, RTLD_NOW);
      if (((PPARFUNDATA) kp->itd[id].stru)->hmody)
        ((PPARFUNDATA) kp->itd[id].stru)->hmody =
          dlopen(((PPARFUNDATA) kp->itd[id].stru)->pathy, RTLD_NOW);
  }
}

void ItemDlg::slotCopy()
{
  int i;

  if ((i = itemList->currentItem()) != -1) {
    kp->nItems++;
    kp->itd = (PITEMDATA) realloc(kp->itd, kp->nItems * sizeof(ITEMDATA));
    copyStru(kp->nItems - 1, i);
    updItemList();
    change = true;
  }
}

void ItemDlg::slotHelp()
{
  kapp->invokeHTMLHelp("kpl/index-5.html", "sec-items");
}

void ItemDlg::slotOK()
{
  done(change);
}

