/****************************************************************/
/*                                                              */
/*      kgraph.h                                                */
/*                                                              */
/*      This file is a part of kpl - a program for graphical    */
/*      presentation of data sets and functions.                */
/*                                                              */
/*      Copyright (C) 1999 Werner Stille                        */
/*      <stille@uni-freiburg.de>                                */
/*                                                              */
/*      Released under the GPL; see file LICENSE for details.   */
/*                                                              */
/****************************************************************/

#ifndef KGRAPH_H
#define KGRAPH_H

#include <qpainter.h>

class KGraph : public QPainter
{
  public:
    KGraph();
    ~KGraph(){};
    int minMaxFl(double *xmin, double *xmax, double *x, int n);
    void expo(double a, int *ie, double *f);
    void expoFl(int *ie, double *f, double *x, int n);
    void expoFn(double (*fkt)(double, double *), int *ie, double *f,
                double xmin, double xmax, double dx, double *p);
    int autoSc(double *xmi, double *xma, double *tic, int *mtic, int *ndig,
               double xmin, double xmax, double fx, double c);
    int autoFl(double *xmi, double *xma, double *tic, int *mtic, int *ndig,
               int *ie, double *fn, double *x, int n, int norm, double c);
    int autoFn(double *ymi, double *yma, double *tic, int * mtic, int *ndig,
               double (*fkt)(double, double *), double xmin, double xmax,
               double dx, double *p, double fn, double c);
    void r2i(double x, double y, int *ix, int *iy);
    int i2cm(int ix, int iy, double *xcm, double *ycm);
    void i2r(int ix, int iy, double fxn, double fyn, double *xr, double *yr);
    void cm2r(double xcm, double ycm, double fxn, double fyn,
         double *xr, double *yr);
    void move(double x, double y);
    void draw(double x, double y);
    void plInit(const QPaintDevice *pd, int offLeft = 0, int offTop = 0);
    void format(double x, double y);
    void setColFrame(const QColor & icol1);
    void setColGrid(const QColor & icol2);
    void setColData(const QColor & icol3);
    void setCol(const QColor & icol1, const QColor & icol2, const QColor & icol3);
    double relsize(double r);
    void Window(double x0, double xl, double y0, double yl);
    void scale(double xl, double xr, double yu, double yo);
    void setdig(int nx, int ny);
    void frame(void);
    void raster(double xtic, double ytic, int intx, int inty, int mode);
    void dir(double ang);
    void text(char *str, int ix, int iy, int iorg);
    void leth(char *str);
    void letx(char* str, int iex);
    void lety(char* str, int iex);
    void symbol(int is);
    void circle(double x0, double y0, double r);
    void arcrel(double x, double y, double r, double angst, double angend);
    void plarry(double *x, double *y, double fx, double fy, int n);
    void plerry(double *x, double *y, double *e, double fx, double fy, int n);
    void plfun(double xfmin, double xfmax, double dx, double fx, double fy,
               double (*fkt)(double, double *), double *p);
    void plpfun(double tmin, double tmax, double dt, double fx, double fy,
                double (*fktx)(double, double *), double *px,
                double (*fkty)(double, double *), double *py);
  protected:
    double factor;
  private:
    int int9(double a);
    void linetype(int is);
    int prenum(double a, int ndig, char *label);
    void letxy(char* str, int iex, int iax);
    int minx, maxx, miny, maxy, kxcur, kycur, ndigx, ndigy, ksymb, minx0,
        miny0;
    float textdir;
    double scalx, scaly, xmin, xmax, ymin, ymax, xcur, ycur, dlen,
           dhite, xscal, yscal, relang, relsin, relcos,
           dash[4], dcur, relsiz;
    QPen pen1, pen2, pen3;
};

#endif
