/****************************************************************/
/*                                                              */
/*      kplwidget.cpp                                           */
/*                                                              */
/*      This file is a part of kpl - a program for graphical    */
/*      presentation of data sets and functions.                */
/*                                                              */
/*      Copyright (C) 1999 Werner Stille                        */
/*      <stille@uni-freiburg.de>                                */
/*                                                              */
/*      Released under the GPL; see file LICENSE for details.   */
/*                                                              */
/****************************************************************/

#include "kplwidget.h"
#include "kpl.h"
#include <qscrollbar.h>

KPlView::KPlView(QWidget *parent, KPl *kpl) : QWidget(parent)
{
  kp = kpl;
  setBackgroundMode(PaletteBase);
  setMouseTracking(TRUE);
}

void KPlView::paintEvent(QPaintEvent *)
{
  kp->paintWidget();
}

void KPlView::mouseMoveEvent(QMouseEvent *e)
{
  setCursor(crossCursor);
  kp->updStatus(e->x(), e->y());
}

KPlWidget::KPlWidget(QWidget *parent) : QWidget(parent)
{
  kp = (KPl *) parent;
  view = new KPlView(this, kp);
  sHScrollPos = sVScrollPos = 0;
  horScrollBar = new QScrollBar(QScrollBar::Horizontal, this);
  connect(horScrollBar, SIGNAL(valueChanged(int)),
          SLOT(slotHorScroll(int)));
  vertScrollBar = new QScrollBar(QScrollBar::Vertical, this);
  connect(vertScrollBar, SIGNAL(valueChanged(int)),
          SLOT(slotVertScroll(int)));
}

KPlWidget::~KPlWidget()
{
  delete horScrollBar;
  delete vertScrollBar;
  delete view;
}

void KPlWidget::slotHorScroll(int value)
{
  sHScrollPos = value;
  view->update();
}

void KPlWidget::slotVertScroll(int value)
{
  sVScrollPos = vertScrollBar->maxValue() - value;
  view->update();
}

void KPlWidget::resizeEvent(QResizeEvent *)
{
  kp->calibrateScrollBars();
}

