/****************************************************************/
/*                                                              */
/*      refreshdlg.cpp                                          */
/*                                                              */
/*      This file is a part of kpl - a program for graphical    */
/*      presentation of data sets and functions.                */
/*                                                              */
/*      Copyright (C) 1999 Werner Stille                        */
/*      <stille@uni-freiburg.de>                                */
/*                                                              */
/*      Released under the GPL; see file LICENSE for details.   */
/*                                                              */
/****************************************************************/

#include "refreshdlg.h"
#include <kapp.h>
#include <kseparator.h>
#include <qlabel.h>
#include <qspinbox.h>
#include <qcheckbox.h>
#include <qpushbutton.h>

RefreshDlg::RefreshDlg(QWidget *parent) : QDialog(parent, 0, TRUE)
{
  setCaption(i18n("Reload periodically"));
  l1 = new QLabel(i18n("Interval"), this);
  l1->setGeometry(10, 10, 63, 22);
  dt = new QSpinBox(1, 600, 1, this);
  dt->setGeometry(75, 10, 60, 22);
  l2 = new QLabel("s", this);
  l2->setGeometry(140, 10, 30, 22);
  activate = new QCheckBox(i18n("Activate"), this);
  activate->setGeometry(75, 50, 112, 22);
  hl = new KSeparator(KSeparator::HLine, this);
  hl->setGeometry(10, 82, 180, 0);
  ok = new QPushButton(i18n("OK"), this);
  ok->setGeometry(10, 92, 85, 25);
  ok->setDefault(TRUE);
  connect(ok, SIGNAL(clicked()), SLOT(accept()));
  cancel = new QPushButton(i18n("Cancel"), this);
  cancel->setGeometry(105, 92, 85, 25);
  connect(cancel, SIGNAL(clicked()), SLOT(reject()));
  adjustSize();
  setFixedSize(size());
}

RefreshDlg::~RefreshDlg()
{
  delete l1;
  delete l2;
  delete dt;
  delete activate;
  delete hl;
  delete ok;
  delete cancel;
}

void RefreshDlg::setWidgets(int tRef, bool act)
{
  dt->setValue(tRef);
  activate->setChecked(act);
}

void RefreshDlg::getValues(int *tRef, bool *act)
{
  *tRef = dt->value();
  *act = activate->isChecked();
}

