/****************************************************************/
/*                                                              */
/*      symboldlg.cpp                                           */
/*                                                              */
/*      This file is a part of kpl - a program for graphical    */
/*      presentation of data sets and functions.                */
/*                                                              */
/*      Copyright (C) 1999 Werner Stille                        */
/*      <stille@uni-freiburg.de>                                */
/*                                                              */
/*      Released under the GPL; see file LICENSE for details.   */
/*                                                              */
/****************************************************************/

#include "symboldlg.h"
#include <kapp.h>
#include <kiconloader.h>
#include <qlabel.h>
#include <qspinbox.h>
#include <qradiobutton.h>
#include <qbuttongroup.h>
#include <qpushbutton.h>

SymbolDlg::SymbolDlg(QWidget *parent, QSpinBox *sb0) :
 QDialog(parent, 0, TRUE)
{
  QRadioButton *rb;
  QString s;
  int i;

  sb = sb0;
  setCaption(i18n("Lines and Markers"));
  g1 = new QGroupBox(i18n("Lines"), this);
  g1->setGeometry(10, 6, 90, 278);
  g2 = new QGroupBox(i18n("Markers"), this);
  g2->setGeometry(112, 6, 160, 278);
  bg = new QButtonGroup();
  for (i = 17; i >= 0; i--) {
    symb[i] = new QRadioButton(s.setNum(-i), this);
    symb[i]->setGeometry(142 + (i / 9) * 76, 28 + (i % 9) * 28, 50, 24);
    bg->insert(symb[i]);
  }
  for (i = 0; i < 9; i++) {
    line[i] = new QRadioButton(s.setNum(i + 1), this);
    line[i]->setGeometry(60, 28 + i * 28, 36, 24);
    bg->insert(line[i]);
    ll[i] = new QLabel(this);
    ll[i]->setGeometry(18, 40 + i * 28, 32, 1);
    ll[i]->setPixmap(kapp->getIconLoader()->loadIcon(s.sprintf("line%i.xpm", i + 1)));
  }
  rb = (QRadioButton *) bg->find(sb->value() + 17);
  rb->setChecked(true);
  for (i = 0; i < 17; i++) {
    ls[i] = new QLabel(this);
    ls[i]->setGeometry(124 + (i / 9) * 76, 36 + (i % 9) * 28, 9, 9);
    ls[i]->setPixmap(kapp->getIconLoader()->loadIcon(s.sprintf("symb%i.xpm", i)));
  }
  ok = new QPushButton(i18n("OK"), this);
  ok->setGeometry(10, 298, 126, 25);
  ok->setDefault(true);
  connect(ok, SIGNAL(clicked()), SLOT(slotOK()));
  cancel = new QPushButton(i18n("Cancel"), this);
  cancel->setGeometry(146, 298, 126, 25);
  connect(cancel, SIGNAL(clicked()), SLOT(reject()));
  adjustSize();
  setFixedSize(size());
}

SymbolDlg::~SymbolDlg()
{
  int i;

  delete g1;
  delete g2;
  for (i = 0; i < 9; i++) {
    delete ll[i];
    delete line[i];
  }
  for (i = 0; i < 17; i++)
    delete ls[i];
  for (i = 0; i < 18; i++)
    delete symb[i];
  delete bg;
  delete ok;
  delete cancel;
}

void SymbolDlg::slotOK()
{
  int i = -1;
  QRadioButton *rb;

  do 
    rb = (QRadioButton *) bg->find(++i);
  while (!rb->isChecked());
  sb->setValue(i - 17);
  accept();
}

